@@ -13,6 +13,7 @@ import datetime as dt
 
 import matplotlib as mpl
 import matplotlib.pyplot as plt
+from matplotlib.ticker import MultipleLocator
 import numpy as np
 
 from metpy.calc import dewpoint_from_relative_humidity
@@ -68,12 +69,11 @@ class Meteogram:
         ln1 = self.ax1.plot(self.dates, ws, label='Wind Speed')
         self.ax1.fill_between(self.dates, ws, 0)
         self.ax1.set_xlim(self.start, self.end)
-        if not plot_range:
-            plot_range = [0, 20, 1]
+        ymin, ymax, ystep = plot_range if plot_range else (0, 20, 2)
         self.ax1.set_ylabel('Wind Speed (knots)', multialignment='center')
-        self.ax1.set_ylim(plot_range[0], plot_range[1], plot_range[2])
-        self.ax1.grid(b=True, which='major', axis='y', color='k', linestyle='--',
-                      linewidth=0.5)
+        self.ax1.set_ylim(ymin, ymax)
+        self.ax1.yaxis.set_major_locator(MultipleLocator(ystep))
+        self.ax1.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)
         ln2 = self.ax1.plot(self.dates, wsmax, '.r', label='3-sec Wind Speed Max')
 
         ax7 = self.ax1.twinx()
@@ -97,22 +97,22 @@ class Meteogram:
             plot_range: Data range for making figure (list of (min,max,step))
         """
         # PLOT TEMPERATURE AND DEWPOINT
-        if not plot_range:
-            plot_range = [10, 90, 2]
+        ymin, ymax, ystep = plot_range if plot_range else (10, 90, 5)
         self.ax2 = fig.add_subplot(4, 1, 2, sharex=self.ax1)
         ln4 = self.ax2.plot(self.dates, t, 'r-', label='Temperature')
         self.ax2.fill_between(self.dates, t, td, color='r')
 
         self.ax2.set_ylabel('Temperature\n(F)', multialignment='center')
-        self.ax2.grid(b=True, which='major', axis='y', color='k', linestyle='--',
-                      linewidth=0.5)
-        self.ax2.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        self.ax2.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)
+        self.ax2.set_ylim(ymin, ymax)
+        self.ax2.yaxis.set_major_locator(MultipleLocator(ystep))
 
         ln5 = self.ax2.plot(self.dates, td, 'g-', label='Dewpoint')
         self.ax2.fill_between(self.dates, td, self.ax2.get_ylim()[0], color='g')
 
         ax_twin = self.ax2.twinx()
-        ax_twin.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        ax_twin.set_ylim(ymin, ymax)
+        ax_twin.yaxis.set_major_locator(MultipleLocator(ystep))
         lines = ln4 + ln5
         labs = [line.get_label() for line in lines]
         ax_twin.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))
@@ -128,20 +128,20 @@ class Meteogram:
             plot_range: Data range for making figure (list of (min,max,step))
         """
         # PLOT RELATIVE HUMIDITY
-        if not plot_range:
-            plot_range = [0, 100, 4]
+        ymin, ymax, ystep = plot_range if plot_range else (0, 100, 5)
         self.ax3 = fig.add_subplot(4, 1, 3, sharex=self.ax1)
         self.ax3.plot(self.dates, rh, 'g-', label='Relative Humidity')
         self.ax3.legend(loc='upper center', bbox_to_anchor=(0.5, 1.22), prop={'size': 12})
-        self.ax3.grid(b=True, which='major', axis='y', color='k', linestyle='--',
-                      linewidth=0.5)
-        self.ax3.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        self.ax3.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)
+        self.ax3.set_ylim(ymin, ymax)
+        self.ax3.yaxis.set_major_locator(MultipleLocator(ystep))
 
         self.ax3.fill_between(self.dates, rh, self.ax3.get_ylim()[0], color='g')
         self.ax3.set_ylabel('Relative Humidity\n(%)', multialignment='center')
         self.ax3.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))
         axtwin = self.ax3.twinx()
-        axtwin.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        axtwin.set_ylim(ymin, ymax)
+        axtwin.yaxis.set_major_locator(MultipleLocator(ystep))
 
     def plot_pressure(self, p, plot_range=None):
         """
@@ -151,21 +151,21 @@ class Meteogram:
             plot_range: Data range for making figure (list of (min,max,step))
         """
         # PLOT PRESSURE
-        if not plot_range:
-            plot_range = [970, 1030, 2]
+        ymin, ymax, ystep = plot_range if plot_range else (970, 1030, 4)
         self.ax4 = fig.add_subplot(4, 1, 4, sharex=self.ax1)
         self.ax4.plot(self.dates, p, 'm', label='Mean Sea Level Pressure')
         self.ax4.set_ylabel('Mean Sea\nLevel Pressure\n(mb)', multialignment='center')
-        self.ax4.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        self.ax4.set_ylim(ymin, ymax)
+        self.ax4.yaxis.set_major_locator(MultipleLocator(ystep))
 
         axtwin = self.ax4.twinx()
-        axtwin.set_ylim(plot_range[0], plot_range[1], plot_range[2])
+        axtwin.set_ylim(ymin, ymax)
+        axtwin.yaxis.set_major_locator(MultipleLocator(ystep))
         axtwin.fill_between(self.dates, p, axtwin.get_ylim()[0], color='m')
         axtwin.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))
 
         self.ax4.legend(loc='upper center', bbox_to_anchor=(0.5, 1.2), prop={'size': 12})
-        self.ax4.grid(b=True, which='major', axis='y', color='k', linestyle='--',
-                      linewidth=0.5)
+        self.ax4.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)
         # OTHER OPTIONAL AXES TO PLOT
         # plot_irradiance
         # plot_precipitation
